/*
  Q Light Controller
  uDMXout.h

  Copyright (C) 2007 Heikki Junnila
                     Lutz (ilLUTZminator)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  Version 2 as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. The license is
  in the file "COPYING".

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef UDMXOUT_H
#define UDMXOUT_H

#include "../common/outputplugin.h"
#include "../../main/src/types.h"


#include <usb.h>    /* this is libusb, see http://libusb.sourceforge.net/ */

#define USBDEV_SHARED_VENDOR    0x16C0  /* VOTI */
#define USBDEV_SHARED_PRODUCT   0x05DC  /* Obdev's free shared PID */
/* Use obdev's generic shared VID/PID pair and follow the rules outlined
 * in firmware/usbdrv/USBID-License.txt.
 */

#define cmd_SetSingleChannel 1
/* usb request for cmd_SetSingleChannel:
	bmRequestType:	ignored by device, should be USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT
	bRequest:		cmd_SetSingleChannel
	wValue:			value of channel to set [0 .. 255]
	wIndex:			channel index to set [0 .. 511]
	wLength:		ignored
*/

#define cmd_SetChannelRange 2
/* usb request for cmd_SetChannelRange:
	bmRequestType:	ignored by device, should be USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT
	bRequest:		cmd_SetChannelRange
	wValue:			number of channels to set [1 .. 512-wIndex]
	wIndex:			index of first channel to set [0 .. 511]
	wLength:		length of data, must be >= wValue
*/


#include <qptrlist.h>
#include <qstring.h>

#define MAXINTERFACES 8


class Configure_uDMXOut;
class QPoint;

extern "C" OutputPlugin* create(t_plugin_id id);
extern "C" void destroy(OutputPlugin* object);

enum DebugLevel {DEBUG_ERROR, DEBUG_WARN, DEBUG_INFO, DEBUG_ALL} ;

class uDMXOut : public OutputPlugin
{
  Q_OBJECT

  friend class Configure_uDMXOut;

 public:
  uDMXOut(t_plugin_id id);
  ~uDMXOut();

  // Plugin methods
  int open();
  int close();
  bool isOpen();
  int configure();
  QString infoText();
  void contextMenu(QPoint pos);

  int setConfigDirectory(QString dir);
  int saveSettings();
  int loadSettings();

  // OutputPlugin methods
  int writeChannel(t_channel channel, t_value value);
  int writeRange(t_channel address, t_value* values, t_channel num);

  int readChannel(t_channel channel, t_value &value);
  int readRange(t_channel address, t_value* values, t_channel num);

  // Own methods
  QString deviceName() { return m_deviceName; }
  void setDeviceName(QString name) { m_deviceName = name; }
  int firstUniverse(){ return m_firstUniverse; }
  int Channels()     { return m_Channels ; }
  int DontSendSame() { return m_DontSendSame ; }
  int Debug()        { return m_Debug ; }

 private slots:
  void slotContextMenuCallback(int item);

 private:
  void activate();
  void createContents(QPtrList <QString> &list);

  int usbStringGet(usb_dev_handle *dev, int index, 
                   int langid, char *buf, int buflen) ;

  bool Debug(int debuglevel, char * format, ...) ;

 private:
  QString m_deviceName;
  QString m_configDir;
  usb_dev_handle *m_device[MAXINTERFACES];

  int m_firstUniverse;
  int m_numOfDevices;
  t_value m_values[KChannelMax];

  int  m_Channels ;
  int  m_DontSendSame ;
  int  m_Debug ;
  int  m_LastInvalidDevice ;
};

#endif
